// Author 		: F. de la Parra
// Date			: January, 2019
// Description 	: This program exercises 2D arrays, ragged arrays,
//				  aliasing and system properties.
//				  
// Version		: 1.0

import java.util.Scanner;
import java.util.Properties;

public class JavaDemo4 {
	
	// Method "main" drives topic selection
	
	public static void main(String[] args) {
		JavaDemo4 demoObject = new JavaDemo4();
		
		// Display menu and wait for a menu item selection
		
		Scanner menuSelect = new Scanner(System.in);
		
		// Initialize menuItem selection.
		int menuItem = 0;
		
		while (menuItem != 9) {
			// Display menu
			System.out.println("\n***JavaDemo3 Menu ***");
			System.out.println("1 - 2D arrays");
			System.out.println("2 - Ragged arrays");
			System.out.println("3 - Aliasing");
			System.out.println("4 - System properties");
			System.out.println("9 - Exit");
			System.out.println();
			System.out.println("Enter demo number:");
			
			// Input a menu selection.
			menuItem = menuSelect.nextInt();
			
			// Run demo corresponding to selected menu item.
			switch (menuItem) {
				
				case 1 : {
					System.out.println("\n*** Running 2D array demo ***\n");
					
					// Execute 2D array demo.
					demoObject.twoDarrayDemo();
					break;
				}
				
				case 2 : {
					System.out.println("\n*** Running ragged array demo ***\n");
					
					// Execute ragged array demo.
					demoObject.raggedArrayDemo();
					break;
				}
				
				case 3 : {
					System.out.println("\n*** Running aliasing demo ***\n");
					
					// Execute aliasing demo.
					demoObject.aliasingDemo();
					break;
				}
		
				case 4 : {
					System.out.println("\n*** Running system properties demo ***\n");
					
					// Execute aliasing demo.
					demoObject.propertiesDemo();
					break;
				}		
			 // case n :     More upcoming demos ...
				
				default : {
					if (menuItem != 9)
						System.out.println("Invalid selection entered");
					break;
				}
			}
			
			if (menuItem != 9) 	
				System.out.println("\n*** Demo completed ***\n");
		}
		
		System.out.println("\n*** Program terminated ***\n");	
	}
	
	// 2D arrays demo.
	
	public void twoDarrayDemo() {
		
		// This demo will show how to determine whether a 2D square
		// table (i.e., a 2D array) is "magic" or not (i.e., the sum
		// of all the elements in any row, column, forward diagonal,
		// reverse diagonal amounts to the same total.
		
		// Initialize variables to compute sums of rows, columns and
		// diagonals.
		int fDiagSum = 0, bDiagSum = 0, rSum = 0, cSum = 0;
		
		// Load a two-dimensional array with a "magic" content.  
		int[][] table =  {{8,1,6},{3,5,7},{4,9,2}};
		
		// Display the length of the array (i.e.,size of first dimension).
		System.out.println("Length of table is " + table.length);
		
		// Arrays to hold the sums on rows, columns and diagonals 
		int[] sums = new int[2 * table.length + 2];
		
		// Index for array "sums".
		int idx = 0;
		
		// Display the length of each first dimension element.
		System.out.println("\nLength of each row of the table:");
		for (int i = 0; i < table.length; i++)
			System.out.println("Lenght of row " + i + " is " + table[i].length);
		
		// Display the content of "table".
		System.out.println("\nContent of array table is:");
		for (int i = 0; i < table[0].length; i++)
			for (int j = 0; j < table[0].length; j++) {
				System.out.print(table[i][j] + " ");
				if (j == table[0].length - 1)
					System.out.println();
			}
			
		// Compute the sum of the forward diagonal.
		for (int i = 0; i < table[0].length; i++)
			fDiagSum += table[i][i];
		System.out.println("\nSum of forward diagonal is " + fDiagSum);
		sums[idx] = fDiagSum;   // Capture sum on forward diagonal.
		idx += 1;
		
		// Compute the sum of the backward diagonal.
		for (int i = 0; i < table[0].length; i++)
			bDiagSum += table[i][table[0].length - i - 1];
		System.out.println("\nSum of backward diagonal is " + bDiagSum);
		sums[idx] = bDiagSum;   // Capture sum on backward diagonal.
		idx += 1;
		
		// Display the sum of each row.
		System.out.println("\nSum of each row of array table is:");
		for (int i = 0; i < table[0].length; i++) {
			rSum = 0;
			System.out.print("Sum of row " + i + " is " );
			for (int j = 0; j < table[0].length; j++) 
				rSum += table[i][j];
			System.out.println(rSum);
			sums[idx] = rSum;   // Capture sum on row.
			idx += 1;
		}
		
		// Display the sum of each colum.
		System.out.println("\nSum for each column of array table is:");
		for (int i = 0; i < table[0].length; i++) {
			cSum = 0;
			System.out.print("Sum of col " + i + " is " );
			for (int j = 0; j < table[0].length; j++) 
				cSum += table[j][i];
			System.out.println(cSum);
			sums[idx] = rSum;   // Capture sum on column.
			idx += 1;
		}
		
		// Check whether table is "magic" or not.
		
		// Assume table is "magic"
		boolean isMagic = true;
		// Initialize "currentSumm" to the value computed for forward diagonal.
		int currentSum = sums[0];
		
		// Compare with remaining sums.
		for (int i = 1; i < sums.length; i++)
			if (currentSum != sums[i]) {
				isMagic = false;
				break;
			}
			
		// Display result of test for "magic" table.
		if (isMagic)
			System.out.println("\nTable is magic!");
		else
			System.out.println("\nTable is not magic!");
	}

	public void raggedArrayDemo() {	 
		// 2D RAGGED ARRAYS DEMO
		
		// Declare a 1D array "raggedTable" (of size 3) of arrays.
		int[][] raggedTable = new int[3][];
		
		// Initialize array in location 0 of the 1D array
		raggedTable[0] = new int[10];
		
		// Load values in elements of array "raggedTable[0]".
		raggedTable[0][0] = 1; 
		raggedTable[0][1] = 6;
		raggedTable[0][2] = 9;
		raggedTable[0][3] = 15;
		raggedTable[0][4] = 20;
		raggedTable[0][5] = 25;
		raggedTable[0][6] = 30;
		raggedTable[0][7] = 50;
		raggedTable[0][8] = 80;
		raggedTable[0][9] = 90;
		
		// Initialize array in location 1 of the 1D array
		raggedTable[1] = new int[2];
		
		// Load values in elements of array "raggedTable[1]".		
		raggedTable[1][0] = 3; 
		raggedTable[1][1] = 5;
		
		// Initialize array in location 2 of the 1D array
		raggedTable[2] = new int[5];
		
		// Load values in elements of array "raggedTable[2]".		
		raggedTable[2][0] = 4; 
		raggedTable[2][1] = 9;
		raggedTable[2][2] = 2;
		raggedTable[2][3] = 10;
		raggedTable[2][4] = 12;
		
		int raggedSum = 0;
		
		// Display the sum of all values for each ragged array.
		System.out.println("\nSums of ragged arrays\n");
		for (int i = 0; i < raggedTable.length; i++) {
			raggedSum = 0;
			for (int j = 0; j < raggedTable[i].length; j++)
				raggedSum += raggedTable[i][j];
			System.out.println("Sum of ragged array " + i + " is " + raggedSum);
		}
	}
	
	// Array aliasing demo.
	
	public void aliasingDemo() {
		Player[] team1 = new Player[26];
		Player[] team2 = new Player[26];
		
		// Create players for team1 (Boston).
		for (int i = 0; i < team1.length; i++)
			team1[i] = new Player(i, "Boston-" + i);
		
		for (int i = 0; i < team2.length; i++)
			team2[i] = new Player(i, "Calgary-" + i);
		
		System.out.println("BEFORE ALIASING");
		
		// Display player 0 from team 2 (Calgary). Before aliasing.
		System.out.println(team2[0].pData());

		// Display player 1 from team 2 (Calgary). Before aliasing.
		System.out.println(team2[1].pData());
		
		team2 = team1;

		System.out.println("\nAFTER ALIASING");
		
		// Display player 0 from team 1 (Boston).
		System.out.println(team1[0].pData());

		// Display player 0 from team 2 (Calgary).
		System.out.println(team1[0].pData());
		
		// Display player 1 from team 1 (Boston).
		System.out.println(team1[1].pData());

		// Display player 1 from team 2 (Calgary).
		System.out.println(team1[1].pData());
	}
	
	// System properties demo.
	
	public void propertiesDemo() {
		// Displaying java System properties.
		Properties props = new Properties(System.getProperties());
		props.list(System.out);
		
		// Displaying single property.
		System.out.println("\nDISPLAY A SPECIFIC PROPERTY");
		System.out.println("Compiler: " + props.getProperty("sun.management.compiler"));
	}
}